/*------------------------------------------------------------------------------
 * MDK Middleware - Component ::USB:Device
 * Copyright (c) 2004-2015 ARM Germany GmbH. All rights reserved.
 *------------------------------------------------------------------------------
 * Name:    USBD_Config_ADC_0.h
 * Purpose: USB Device Audio Device Class (ADC) Configuration
 * Rev.:    V5.2
 *----------------------------------------------------------------------------*/

//-------- <<< Use Configuration Wizard in Context Menu >>> --------------------

// <h>USB Device: Audio Device Class (ADC) 0
//   <o>Assign Device Class to USB Device # <0-3>
//   <i>Select USB Device that is used for this Device Class instance
#define USBD_ADC0_DEV                       0

//   <h>Isochronous Endpoint Settings
//   <i>By default, the settings match the first USB Class instance in a USB Device.
//   <i>Endpoint conflicts are flagged by compile-time error messages.

//     <e>Isochronous OUT Endpoint Settings
#define USBD_ADC0_EP_ISO_OUT_EN             1

//       <o.0..3>Isochronous OUT Endpoint Number
//                 <1=>1   <2=>2   <3=>3   <4=>4   <5=>5   <6=>6   <7=>7
//         <8=>8   <9=>9   <10=>10 <11=>11 <12=>12 <13=>13 <14=>14 <15=>15
#define USBD_ADC0_EP_ISO_OUT                1

//       <h>Endpoint Settings
//         <i>Parameters are used to create USB Descriptors and for memory
//         <i>allocation in the USB component.
//
//         <h>Full/Low-speed (High-speed disabled)
//         <i>Parameters apply when High-speed is disabled in USBD_Config_n.c
//           <o.0..9>Maximum Endpoint Packet Size (in bytes) <0-1023>
//           <i>Specifies the physical packet size used for information exchange.
//           <i>Maximum value is 1023.
#define USBD_ADC0_EP_ISO_OUT_WMAXPACKETSIZE 64

//           <o.0..4>Endpoint polling Interval (in ms)
//           <i>Specifies the frequency of requests initiated by USB Host for
//           <i>getting audio samples data.
//             <1=>    1 <2=>    2 <3=>     4 <4=>     8
//             <5=>   16 <6=>   32 <7=>    64 <8=>   128
//             <9=>  256 <10=> 512 <11=> 1024 <12=> 2048
//             <13=>4096 <14=>8192 <15=>16384 <16=>32768
#define USBD_ADC0_EP_ISO_OUT_BINTERVAL      1

//         </h>
//       </h>
//     </e>

//     <e>Isochronous IN Endpoint Settings
#define USBD_ADC0_EP_ISO_IN_EN              1

//       <o.0..3>Isochronous IN Endpoint Number
//                 <1=>1   <2=>2   <3=>3   <4=>4   <5=>5   <6=>6   <7=>7
//         <8=>8   <9=>9   <10=>10 <11=>11 <12=>12 <13=>13 <14=>14 <15=>15
#define USBD_ADC0_EP_ISO_IN                 2

//       <h>Endpoint Settings
//         <i>Parameters are used to create USB Descriptors and for memory
//         <i>allocation in the USB component.
//
//         <h>Full/Low-speed (High-speed disabled)
//         <i>Parameters apply when High-speed is disabled in USBD_Config_n.c
//           <o.0..9>Maximum Endpoint Packet Size (in bytes) <0-1023>
//           <i>Specifies the physical packet size used for information exchange.
//           <i>Maximum value is 1023.
#define USBD_ADC0_EP_ISO_IN_WMAXPACKETSIZE  32

//           <o.0..4>Endpoint polling Interval (in ms)
//           <i>Specifies the frequency of requests initiated by USB Host for
//           <i>getting audio samples data.
//             <1=>    1 <2=>    2 <3=>     4 <4=>     8
//             <5=>   16 <6=>   32 <7=>    64 <8=>   128
//             <9=>  256 <10=> 512 <11=> 1024 <12=> 2048
//             <13=>4096 <14=>8192 <15=>16384 <16=>32768
#define USBD_ADC0_EP_ISO_IN_BINTERVAL       1

//         </h>
//       </h>
//     </e>
//   </h>

//   <h>Audio Device Class Settings
//   <i>Parameters are used to create USB Descriptors and for memory allocation
//   <i>in the USB component.
//
//     <s.126>Audio Control Interface String
#define USBD_ADC0_CIF_STR_DESC              L"USB_ADC0"

//     <h>Audio Output Settings (Speaker)

//       <s.126>Audio Streaming (Zero Bandwidth) Interface String
#define USBD_ADC0_OUT_SIF_STR0_DESC         L"USB_ADC0_SPKR_OFF"

//       <s.126>Audio Streaming (Operational) Interface String
#define USBD_ADC0_OUT_SIF_STR1_DESC         L"USB_ADC0_SPKR_ON"

//       <o.0..1>Channel Configuration <1=>Mono <2=>Stereo
#define USBD_ADC0_OUT_CH_NUM                2

//       <o.0..23>Sample Frequency (in Hz) <0-16777215>
#define USBD_ADC0_OUT_TSAM_FREQ             16000

//       <o.0..7>Audio Subframe Size <1=> 1 byte <2=> 2 bytes <3=> 3 bytes <4=> 4 bytes
//       <i>Configures size of sample data in bytes.
#define USBD_ADC0_OUT_BSUBFRAMESIZE         2

//       <o.0..7>Sample Resolution <8=> 8 bits <16=> 16 bits <24=> 24 bits <32=> 32 bits
#define USBD_ADC0_OUT_BBITRESOLUTION        16

//       <o>Buffer Size (in samples) <1-16777215>
#define USBD_ADC0_OUT_BUF_SIZE              2048

//     </h>

//     <h>Audio Input Settings (Microphone)

//       <s.126>Audio Streaming (Zero Bandwidth) Interface String
#define USBD_ADC0_IN_SIF_STR0_DESC          L"USB_ADC0_MIC_OFF"

//       <s.126>Audio Streaming (Operational) Interface String
#define USBD_ADC0_IN_SIF_STR1_DESC          L"USB_ADC0_MIC_ON"

//       <o.0..1>Channel Configuration<1=>Mono <2=>Stereo
#define USBD_ADC0_IN_CH_NUM                 1

//       <o.0..23>Sample Frequency (in Hz) <0-16777215>
#define USBD_ADC0_IN_TSAM_FREQ              16000

//       <o.0..7>Audio Subframe Size <1=> 1 byte <2=> 2 bytes <3=> 3 bytes <4=> 4 bytes
//       <i>Configures size of sample data in bytes.
#define USBD_ADC0_IN_BSUBFRAMESIZE          2

//       <o.0..7>Sample Resolution <8=> 8 bits <16=> 16 bits <24=> 24 bits <32=> 32 bits
#define USBD_ADC0_IN_BBITRESOLUTION         16

//       <o>Buffer Size (in samples) <1-16777215>
#define USBD_ADC0_IN_BUF_SIZE               4096

//     </h>
//   </h>

//   <h>OS Resources Settings
//   <i>These settings are used to optimize usage of OS resources.
//     <o>Audio Device Class Thread Stack Size <64-65536>
#define USBD_ADC0_THREAD_STACK_SIZE         1024

//        Audio Device Class Thread Priority
#define USBD_ADC0_THREAD_PRIORITY           osPriorityAboveNormal

//   </h>
// </h>
