//*****************************************************************************
// LPC5410x image-header, header file
//
//*****************************************************************************
//
// Copyright(C) NXP Semiconductors, 2014-15
// All rights reserved.
//
// Software that is described herein is for illustrative purposes only
// which provides customers with programming information regarding the
// LPC products.  This software is supplied "AS IS" without any warranties of
// any kind, and NXP Semiconductors and its licensor disclaim any and
// all warranties, express or implied, including all implied warranties of
// merchantability, fitness for a particular purpose and non-infringement of
// intellectual property rights.  NXP Semiconductors assumes no responsibility
// or liability for the use of the software, conveys no license or rights under any
// patent, copyright, mask work right, or any other intellectual property rights in
// or to any products. NXP Semiconductors reserves the right to make changes
// in the software without notification. NXP Semiconductors also makes no
// representation or warranty that such application will be suitable for the
// specified use without further testing or modification.
//
// Permission to use, copy, modify, and distribute this software and its
// documentation is hereby granted, under NXP Semiconductors' and its
// licensor's relevant copyrights in the software, without fee, provided that it
// is used in conjunction with NXP Semiconductors microcontrollers.  This
// copyright, permission, and disclaimer notice must appear in all copies of
// this code.
//*****************************************************************************

// AWB
#include <stdint.h>

#define IMAGE_HDR_MARKER			0xFEEDA5A5									// image header marker
#define IMAGE_LEGACY_SIG			0x00000000									// legacy image signature
#define IMAGE_SINGLE_ENH_SIG		0xEDDC9494									// single enhanced image signature
#define IMAGE_DUAL_ENH_SIG			0x0FFEB6B6									// dual enhanced image signature

//------------------------------------------------------------------------------
//	Configure the selected image signature:
//	This value is set in the "Symbols" settings in the project properties
//------------------------------------------------------------------------------

#if defined (__ECRP_SIMPLE)
#define IMAGE_SIG		IMAGE_LEGACY_SIG
#else
#if !defined (__ECRP_DUAL_IMAGE)
#define IMAGE_SIG		IMAGE_SINGLE_ENH_SIG
#else
#define IMAGE_SIG		IMAGE_DUAL_ENH_SIG
#endif
#endif

typedef enum {
	IMG_NORMAL = 0,																// Normal image with CRC.
	IMG_NO_CRC																	// No CRC check required. Used during development.
} SL_IMAGE_T;

//------------------------------------------------------------------------------
//	Image header structure:
//	- All application images should define this structure at of offset 0x100.
//	- Adding it to the start-up file is recommended.
//------------------------------------------------------------------------------
typedef struct _IMG_HEADER_T {
	uint32_t	header_marker;													// Image header marker should always be set to 0xFEEDA5A5
	SL_IMAGE_T	img_type;														// Image check type, with or without optional CRC
	uint32_t	reserved;														// Reserved value (compatibility with Niobe)
	uint32_t	img_len;														// Image length or the length of image CRC check should be done.
	uint32_t	crc_value;														// CRC value
	uint32_t	version;														// Image version for multi-image support
} IMG_HEADER_T;

