/*
 * @brief LPC5460x OTP driver
 *
 * @note
 * Copyright(C) NXP Semiconductors, 2016
 * All rights reserved.
 *
 * @par
 * Software that is described herein is for illustrative purposes only
 * which provides customers with programming information regarding the
 * LPC products.  This software is supplied "AS IS" without any warranties of
 * any kind, and NXP Semiconductors and its licensor disclaim any and
 * all warranties, express or implied, including all implied warranties of
 * merchantability, fitness for a particular purpose and non-infringement of
 * intellectual property rights.  NXP Semiconductors assumes no responsibility
 * or liability for the use of the software, conveys no license or rights under any
 * patent, copyright, mask work right, or any other intellectual property rights in
 * or to any products. NXP Semiconductors reserves the right to make changes
 * in the software without notification. NXP Semiconductors also makes no
 * representation or warranty that such application will be suitable for the
 * specified use without further testing or modification.
 *
 * @par
 * Permission to use, copy, modify, and distribute this software and its
 * documentation is hereby granted, under NXP Semiconductors' and its
 * licensor's relevant copyrights in the software, without fee, provided that it
 * is used in conjunction with NXP Semiconductors microcontrollers.  This
 * copyright, permission, and disclaimer notice must appear in all copies of
 * this code.
 */

#ifndef _OTP_5460X_H_
#define _OTP_5460X_H_

#ifdef __cplusplus
extern "C" {
#endif

/** @defgroup OTP_5460X CHIP: LPC5460X OTP driver
 * @ingroup CHIP_5460X_Drivers
 * @{
 */

/** OTP Bank Count */
#define OTP_BANK_COUNT      4

#define OTP_AES1_BANK       1
#define OTP_AES2_BANK       2
#define OTP_ROM_BANK        3

#define OTP_ECRP_REG        0	/* Register used to contain eCRP bits */
#define OTP_SECURE_REG      1	/* Register used to contain security feature bits */

/**
 * @brief OTP register block structure
 */
typedef struct {
	__I uint32_t   OTP[OTP_BANK_COUNT][4];			/* 0x0 - OTP[0][0] .. OTP[3][3] */
} LPC_OTP_T;


/**
 * @brief	Returns the reqister value from bank[regOffset]
 * @param	bank		: Bank to read from 0-3
 * @param   regOffset	: Register offset to read 0-3
 * @return	Value at specified location.
 */
static INLINE uint32_t Chip_OTP_ReadBankRegister(uint32_t bank, uint32_t regOffset)
{
	return LPC_OTP->OTP[bank][regOffset];
}


/**
 * @}
 */

#ifdef __cplusplus
}
#endif

#endif /* _OTP_5460X_H_ */
