/*
 * @brief LPC5460x OTP ROM API
 *
 * @note
 * Copyright(C) NXP Semiconductors, 2016
 * All rights reserved.
 *
 * @par
 * Software that is described herein is for illustrative purposes only
 * which provides customers with programming information regarding the
 * LPC products.  This software is supplied "AS IS" without any warranties of
 * any kind, and NXP Semiconductors and its licensor disclaim any and
 * all warranties, express or implied, including all implied warranties of
 * merchantability, fitness for a particular purpose and non-infringement of
 * intellectual property rights.  NXP Semiconductors assumes no responsibility
 * or liability for the use of the software, conveys no license or rights under any
 * patent, copyright, mask work right, or any other intellectual property rights in
 * or to any products. NXP Semiconductors reserves the right to make changes
 * in the software without notification. NXP Semiconductors also makes no
 * representation or warranty that such application will be suitable for the
 * specified use without further testing or modification.
 *
 * @par
 * Permission to use, copy, modify, and distribute this software and its
 * documentation is hereby granted, under NXP Semiconductors' and its
 * licensor's relevant copyrights in the software, without fee, provided that it
 * is used in conjunction with NXP Semiconductors microcontrollers.  This
 * copyright, permission, and disclaimer notice must appear in all copies of
 * this code.
 */

#ifndef __OTP_ROM_API_H_
#define __OTP_ROM_API_H_


typedef struct _OTPD_API {
	ErrorCode_t (*otpInit)(void);
	ErrorCode_t (*otpEnableBankWriteMask)(uint32_t bankMask);
	ErrorCode_t (*otpDisableBankWriteMask)(uint32_t bankMask);
	ErrorCode_t (*otpEnableBankWriteLock)(uint32_t bankIndex, uint32_t regEnableMask, uint32_t regDisableMask,
										  uint32_t lockWrite);
	ErrorCode_t (*otpEnableBankReadLock)(uint32_t bankIndex, uint32_t regEnableMask, uint32_t regDisableMask,
										 uint32_t lockWrite);
	ErrorCode_t (*otpProgramReg)(uint32_t bankIndex, uint32_t regIndex, uint32_t value);
	ErrorCode_t (*otpProgramBank)(uint32_t bankIndex, uint32_t valueArray[4]);
	ErrorCode_t (*otpEnableUniqueKeyWriteLock)(uint32_t regEnable, uint32_t lockWrite);
	ErrorCode_t (*otpProgramUniqueKey)(uint32_t valueArray[3]);
	ErrorCode_t (*otpSetupShuffler)(uint32_t dirIn, uint32_t shuffleEn);
	ErrorCode_t (*otpProgramUserKey)(uint32_t programEn);
	uint32_t    (*rngRead)(void);
	uint32_t    (*otpGetDriverVersion)(void);
} ROM_OTP_API_T;

#endif /* __OTP_ROM_API_H_ */

