;/*
; * @brief LPC5460x startup code for Keil
; *
; * @note
; * Copyright(C) NXP Semiconductors, 2014
; * All rights reserved.
; *
; * @par
; * Software that is described herein is for illustrative purposes only
; * which provides customers with programming information regarding the
; * LPC products.  This software is supplied "AS IS" without any warranties of
; * any kind, and NXP Semiconductors and its licensor disclaim any and
; * all warranties, express or implied, including all implied warranties of
; * merchantability, fitness for a particular purpose and non-infringement of
; * intellectual property rights.  NXP Semiconductors assumes no responsibility
; * or liability for the use of the software, conveys no license or rights under any
; * patent, copyright, mask work right, or any other intellectual property rights in
; * or to any products. NXP Semiconductors reserves the right to make changes
; * in the software without notification. NXP Semiconductors also makes no
; * representation or warranty that such application will be suitable for the
; * specified use without further testing or modification.
; *
; * @par
; * Permission to use, copy, modify, and distribute this software and its
; * documentation is hereby granted, under NXP Semiconductors' and its
; * licensor's relevant copyrights in the software, without fee, provided that it
; * is used in conjunction with NXP Semiconductors microcontrollers.  This
; * copyright, permission, and disclaimer notice must appear in all copies of
; * this code.
; */

; <h> Stack Configuration
;   <o> Stack Size (in Bytes) <0x0-0xFFFFFFFF:8>
; </h>

Stack_Size      EQU     0x00000200

                AREA    STACK, NOINIT, READWRITE, ALIGN=3
Stack_Mem       SPACE   Stack_Size
__initial_sp


; <h> Heap Configuration
;   <o>  Heap Size (in Bytes) <0x0-0xFFFFFFFF:8>
; </h>

Heap_Size       EQU     0x00000100

                AREA    HEAP, NOINIT, READWRITE, ALIGN=3
__heap_base
Heap_Mem        SPACE   Heap_Size
__heap_limit

                PRESERVE8
                THUMB

; Vector Table Mapped to Address 0 at Reset
                AREA    RESET, DATA, READONLY
                EXPORT  __Vectors

__Vectors       DCD     __initial_sp              ; Top of Stack
                DCD     Reset_Handler             ; Reset Handler

                DCD     NMI_Handler
                DCD     HardFault_Handler
                DCD     MemManage_Handler
                DCD     BusFault_Handler
                DCD     UsageFault_Handler
__vector_table_0x1c
				DCD     0                         ; Checksum of the first 7 words
				IF		:DEF:NO_CRP
				DCD		0xFFFFFFFF
				ELSE
				DCD     ECRP_VAL
				ENDIF
				DCD     0                         ; Enhanced image marker, set to 0x0 for legacy boot
				DCD     0                         ; Pointer to enhanced boot block, set to 0x0 for legacy boot
				DCD     SVC_Handler
				DCD     DebugMon_Handler
				DCD     0
				DCD     PendSV_Handler
				DCD     SysTick_Handler

        ; External Interrupts
				DCD		WDT_BOD_IRQHandler          ; 0, Watchdog, BOD, and FLASH are grouped now.
				DCD		DMA_IRQHandler              ; 1, DMA Controller
				DCD		GINT0_IRQHandler            ; 2, GPIO Group0 Interrupt
				DCD		GINT1_IRQHandler            ; 3, GPIO Group1 Interrupt
				DCD		PIN_INT0_IRQHandler         ; 4, PIO INT0
				DCD		PIN_INT1_IRQHandler         ; 5, PIO INT1
				DCD		PIN_INT2_IRQHandler         ; 6, PIO INT2
				DCD		PIN_INT3_IRQHandler         ; 7, PIO INT3
				DCD		UTICK_IRQHandler            ; 8, UTICK timer
				DCD		MRT_IRQHandler              ; 9, Multi-Rate Timer
				DCD		CT32B0_IRQHandler           ; 10, TIMER0
				DCD		CT32B1_IRQHandler           ; 11, TIMER1
				DCD		SCT0_IRQHandler             ; 12, Smart Counter Timer
				DCD		CT32B3_IRQHandler           ; 13, TIMER3
				DCD		FLEXCOMM0_IRQHandler        ; 14, FLEX Comm0
				DCD		FLEXCOMM1_IRQHandler        ; 15, FLEX Comm1
				DCD		FLEXCOMM2_IRQHandler        ; 16, FLEX Comm2
				DCD		FLEXCOMM3_IRQHandler        ; 17, FLEX Comm3
				DCD		FLEXCOMM4_IRQHandler        ; 18, FLEX Comm4
				DCD		FLEXCOMM5_IRQHandler        ; 19, FLEX Comm5
				DCD		FLEXCOMM6_IRQHandler        ; 20, FLEX Comm6
				DCD		FLEXCOMM7_IRQHandler        ; 21, FLEX Comm7
				DCD		ADC_SEQA_IRQHandler         ; 22, ADC0 A sequence (A/D Converter) interrupt
				DCD		ADC_SEQB_IRQHandler         ; 23, ADC0 B sequence (A/D Converter) interrupt
				DCD		ADC_THCMP_IRQHandler        ; 24, ADC THCMP and OVERRUN ORed
				DCD		DMIC_IRQHandler             ; 25, Audio DMIC
				DCD		HWVAD_IRQHandler            ; 26, Audio HWVAD
				DCD		USB0_NEEDCLK_IRQHandler     ; 27, USB0 NeedClk or Wakeup
				DCD		USB0_IRQHandler             ; 28, USB0
				DCD		RTC_IRQHandler              ; 29, RTC Timer
				DCD		IOH_IRQHandler              ; 30, IOH
				DCD		MAILBOX_WAKEUP_IRQHandler   ; 31, Mailbox and GPIO wakeup are grouped

				DCD		PIN_INT4_IRQHandler         ; 32, PIO INT4
				DCD		PIN_INT5_IRQHandler         ; 33, PIO INT5
				DCD		PIN_INT6_IRQHandler         ; 34, PIO INT6
				DCD		PIN_INT7_IRQHandler         ; 35, PIO INT7
				DCD		CT32B2_IRQHandler           ; 36, TIMER2
				DCD		CT32B4_IRQHandler           ; 37, TIMER4
				DCD		RITIMER_IRQHandler          ; 38, OS Timer
				DCD		SPIFI_IRQHandler            ; 39, SPIFI             ;
				DCD		FLEXCOMM8_IRQHandler        ; 40, FLEX Comm8
				DCD		FLEXCOMM9_IRQHandler        ; 41, FLEX Comm9
				DCD		SDIO_IRQHandler             ; 42, SDIO
				DCD		CAN0_0_IRQHandler           ; 43, CAN0 INT0
				DCD		CAN0_1_IRQHandler           ; 44, CAN0 INT1
				DCD		CAN1_0_IRQHandler           ; 45, CAN1 INT0
				DCD		CAN1_1_IRQHandler           ; 46, CAN1 INT1
				DCD		USB1_IRQHandler             ; 47, USB1
				DCD		USB1_NEEDCLK_IRQHandler     ; 48, USB1NeedClk
				DCD		ETHERNET_IRQHandler         ; 49, Ethernet
				DCD		ETHERNET_PMT_IRQHandler     ; 50, Ethernet PMT
				DCD		ETHERNET_MACLP_IRQHandler   ; 51, Ethernet MACLP
				DCD		EEPROM_IRQHandler           ; 52, EEPROM
				DCD		LCD_IRQHandler              ; 53, LCD
				DCD		SHA_IRQHandler              ; 54, SHA

				DCD		SMARTCARD0_IRQHandler       ; 55, Smartcard 0
				DCD		SMARTCARD1_IRQHandler       ; 56, Smartcard 1

				AREA	|.text|, CODE, READONLY

; Reset Handler - shared for both cores
Reset_Handler   PROC
                EXPORT	Reset_Handler				[WEAK]
                EXPORT	SystemInit					[WEAK]
                IMPORT	__main

				LDR		r0, =SystemInit
				BLX		r0
				LDR		r0, =__main
				BX		r0
				ENDP

; For cores with SystemInit() or __main(), the code will sleep the MCU
SystemInit		PROC
                EXPORT  SystemInit					[WEAK]
				BX		lr
				ENDP

; Dummy Exception Handlers (infinite loops which can be modified)
NMI_Handler     PROC
                EXPORT  NMI_Handler               [WEAK]
                B       .
                ENDP
HardFault_Handler\
                PROC
                EXPORT  HardFault_Handler         [WEAK]
                B       .
                ENDP
MemManage_Handler     PROC
                EXPORT  MemManage_Handler         [WEAK]
                B       .
                ENDP
BusFault_Handler PROC
                EXPORT  BusFault_Handler          [WEAK]
                B       .
                ENDP
UsageFault_Handler PROC
                EXPORT  UsageFault_Handler        [WEAK]
                B       .
                ENDP
SVC_Handler     PROC
                EXPORT  SVC_Handler               [WEAK]
                B       .
                ENDP
DebugMon_Handler PROC
                EXPORT  DebugMon_Handler               [WEAK]
                B       .
                ENDP
PendSV_Handler  PROC
                EXPORT  PendSV_Handler            [WEAK]
                B       .
                ENDP
SysTick_Handler PROC
                EXPORT  SysTick_Handler           [WEAK]
                B       .
                ENDP

Default_Handler PROC
				EXPORT		WDT_BOD_IRQHandler          [WEAK] ; 0, Watchdog, BOD, and FLASH are grouped now.
				EXPORT		DMA_IRQHandler              [WEAK] ; 1, DMA Controller
				EXPORT		GINT0_IRQHandler            [WEAK] ; 2, GPIO Group0 Interrupt
				EXPORT		GINT1_IRQHandler            [WEAK] ; 3, GPIO Group1 Interrupt
				EXPORT		PIN_INT0_IRQHandler         [WEAK] ; 4, PIO INT0
				EXPORT		PIN_INT1_IRQHandler         [WEAK] ; 5, PIO INT1
				EXPORT		PIN_INT2_IRQHandler         [WEAK] ; 6, PIO INT2
				EXPORT		PIN_INT3_IRQHandler         [WEAK] ; 7, PIO INT3
				EXPORT		UTICK_IRQHandler            [WEAK] ; 8, UTICK timer
				EXPORT		MRT_IRQHandler              [WEAK] ; 9, Multi-Rate Timer
				EXPORT		CT32B0_IRQHandler           [WEAK] ; 10, TIMER0
				EXPORT		CT32B1_IRQHandler           [WEAK] ; 11, TIMER1
				EXPORT		SCT0_IRQHandler             [WEAK] ; 12, Smart Counter Timer
				EXPORT		CT32B3_IRQHandler           [WEAK] ; 13, TIMER3
				EXPORT		FLEXCOMM0_IRQHandler        [WEAK] ; 14, FLEX Comm0
				EXPORT		FLEXCOMM1_IRQHandler        [WEAK] ; 15, FLEX Comm1
				EXPORT		FLEXCOMM2_IRQHandler        [WEAK] ; 16, FLEX Comm2
				EXPORT		FLEXCOMM3_IRQHandler        [WEAK] ; 17, FLEX Comm3
				EXPORT		FLEXCOMM4_IRQHandler        [WEAK] ; 18, FLEX Comm4
				EXPORT		FLEXCOMM5_IRQHandler        [WEAK] ; 19, FLEX Comm5
				EXPORT		FLEXCOMM6_IRQHandler        [WEAK] ; 20, FLEX Comm6
				EXPORT		FLEXCOMM7_IRQHandler        [WEAK] ; 21, FLEX Comm7
				EXPORT		ADC_SEQA_IRQHandler         [WEAK] ; 22, ADC0 A sequence (A/D Converter) interrupt
				EXPORT		ADC_SEQB_IRQHandler         [WEAK] ; 23, ADC0 B sequence (A/D Converter) interrupt
				EXPORT		ADC_THCMP_IRQHandler        [WEAK] ; 24, ADC THCMP and OVERRUN ORed
				EXPORT		DMIC_IRQHandler             [WEAK] ; 25, Audio DMIC
				EXPORT		HWVAD_IRQHandler            [WEAK] ; 26, Audio HWVAD
				EXPORT		USB0_NEEDCLK_IRQHandler     [WEAK] ; 27, USB0 NeedClk or Wakeup
				EXPORT		USB0_IRQHandler             [WEAK] ; 28, USB0
				EXPORT		RTC_IRQHandler              [WEAK] ; 29, RTC Timer
				EXPORT		IOH_IRQHandler              [WEAK] ; 30, IOH
				EXPORT		MAILBOX_WAKEUP_IRQHandler   [WEAK] ; 31, Mailbox and GPIO wakeup are grouped

				EXPORT		PIN_INT4_IRQHandler         [WEAK] ; 32, PIO INT4
				EXPORT		PIN_INT5_IRQHandler         [WEAK] ; 33, PIO INT5
				EXPORT		PIN_INT6_IRQHandler         [WEAK] ; 34, PIO INT6
				EXPORT		PIN_INT7_IRQHandler         [WEAK] ; 35, PIO INT7
				EXPORT		CT32B2_IRQHandler           [WEAK] ; 36, TIMER2
				EXPORT		CT32B4_IRQHandler           [WEAK] ; 37, TIMER4
				EXPORT		RITIMER_IRQHandler          [WEAK] ; 38, OS Timer
				EXPORT		SPIFI_IRQHandler            [WEAK] ; 39, SPIFI             ;
				EXPORT		FLEXCOMM8_IRQHandler        [WEAK] ; 40, FLEX Comm8
				EXPORT		FLEXCOMM9_IRQHandler        [WEAK] ; 41, FLEX Comm9
				EXPORT		SDIO_IRQHandler             [WEAK] ; 42, SDIO
				EXPORT		CAN0_0_IRQHandler           [WEAK] ; 43, CAN0 INT0
				EXPORT		CAN0_1_IRQHandler           [WEAK] ; 44, CAN0 INT1
				EXPORT		CAN1_0_IRQHandler           [WEAK] ; 45, CAN1 INT0
				EXPORT		CAN1_1_IRQHandler           [WEAK] ; 46, CAN1 INT1
				EXPORT		USB1_IRQHandler             [WEAK] ; 47, USB1
				EXPORT		USB1_NEEDCLK_IRQHandler     [WEAK] ; 48, USB1NeedClk
				EXPORT		ETHERNET_IRQHandler         [WEAK] ; 49, Ethernet
				EXPORT		ETHERNET_PMT_IRQHandler     [WEAK] ; 50, Ethernet PMT
				EXPORT		ETHERNET_MACLP_IRQHandler   [WEAK] ; 51, Ethernet MACLP
				EXPORT		EEPROM_IRQHandler           [WEAK] ; 52, EEPROM
				EXPORT		LCD_IRQHandler              [WEAK] ; 53, LCD
				EXPORT		SHA_IRQHandler              [WEAK] ; 54, SHA

				EXPORT		SMARTCARD0_IRQHandler       [WEAK] ; 55, Smartcard 0
				EXPORT		SMARTCARD1_IRQHandler       [WEAK] ; 56, Smartcard 1

WDT_BOD_IRQHandler
DMA_IRQHandler
GINT0_IRQHandler
GINT1_IRQHandler
PIN_INT0_IRQHandler
PIN_INT1_IRQHandler
PIN_INT2_IRQHandler
PIN_INT3_IRQHandler
UTICK_IRQHandler
MRT_IRQHandler
CT32B0_IRQHandler
CT32B1_IRQHandler
SCT0_IRQHandler
CT32B3_IRQHandler
FLEXCOMM0_IRQHandler
FLEXCOMM1_IRQHandler
FLEXCOMM2_IRQHandler
FLEXCOMM3_IRQHandler
FLEXCOMM4_IRQHandler
FLEXCOMM5_IRQHandler
FLEXCOMM6_IRQHandler
FLEXCOMM7_IRQHandler
ADC_SEQA_IRQHandler
ADC_SEQB_IRQHandler
ADC_THCMP_IRQHandler
DMIC_IRQHandler
HWVAD_IRQHandler
USB0_NEEDCLK_IRQHandler
USB0_IRQHandler
RTC_IRQHandler
IOH_IRQHandler
MAILBOX_WAKEUP_IRQHandler

PIN_INT4_IRQHandler
PIN_INT5_IRQHandler
PIN_INT6_IRQHandler
PIN_INT7_IRQHandler
CT32B2_IRQHandler
CT32B4_IRQHandler
RITIMER_IRQHandler
SPIFI_IRQHandler
FLEXCOMM8_IRQHandler
FLEXCOMM9_IRQHandler
SDIO_IRQHandler
CAN0_0_IRQHandler
CAN0_1_IRQHandler
CAN1_0_IRQHandler
CAN1_1_IRQHandler
USB1_IRQHandler
USB1_NEEDCLK_IRQHandler
ETHERNET_IRQHandler
ETHERNET_PMT_IRQHandler
ETHERNET_MACLP_IRQHandler
EEPROM_IRQHandler
LCD_IRQHandler
SHA_IRQHandler

SMARTCARD0_IRQHandler
SMARTCARD1_IRQHandler

                B       .

                ENDP


                ALIGN


; User Initial Stack & Heap

                IF      :DEF:__MICROLIB

                EXPORT  __initial_sp
                EXPORT  __heap_base
                EXPORT  __heap_limit

                ELSE

                IMPORT  __use_two_region_memory
                EXPORT  __user_initial_stackheap
__user_initial_stackheap

                LDR     R0, =  Heap_Mem
                LDR     R1, =(Stack_Mem + Stack_Size)
                LDR     R2, = (Heap_Mem +  Heap_Size)
                LDR     R3, = Stack_Mem
                BX      LR

                ALIGN

                ENDIF


                END
