/*
 * @brief OTP example using ROM API
 *
 * @note
 * Copyright(C) NXP Semiconductors, 2016
 * All rights reserved.
 *
 * @par
 * Software that is described herein is for illustrative purposes only
 * which provides customers with programming information regarding the
 * LPC products.  This software is supplied "AS IS" without any warranties of
 * any kind, and NXP Semiconductors and its licensor disclaim any and
 * all warranties, express or implied, including all implied warranties of
 * merchantability, fitness for a particular purpose and non-infringement of
 * intellectual property rights.  NXP Semiconductors assumes no responsibility
 * or liability for the use of the software, conveys no license or rights under any
 * patent, copyright, mask work right, or any other intellectual property rights in
 * or to any products. NXP Semiconductors reserves the right to make changes
 * in the software without notification. NXP Semiconductors also makes no
 * representation or warranty that such application will be suitable for the
 * specified use without further testing or modification.
 *
 * @par
 * Permission to use, copy, modify, and distribute this software and its
 * documentation is hereby granted, under NXP Semiconductors' and its
 * licensor's relevant copyrights in the software, without fee, provided that it
 * is used in conjunction with NXP Semiconductors microcontrollers.  This
 * copyright, permission, and disclaimer notice must appear in all copies of
 * this code.
 */

#include "board.h"

/*****************************************************************************
 * Private types/enumerations/variables
 ****************************************************************************/


/*****************************************************************************
 * Public types/enumerations/variables
 ****************************************************************************/

/*****************************************************************************
 * Private functions
 ****************************************************************************/

/*****************************************************************************
 * Public functions
 ****************************************************************************/

/* Main application entry point */
int main(void)
{
	int loop = 1;	/* Used to fix the unreachable statement warning */
	uint32_t ecrp_otp_val, secure_otp_val;
	ROM_OTP_API_T *otpApi = (ROM_OTP_API_T *)(LPC_ROM_API->otpApiBase);

	SystemCoreClockUpdate();
	Board_Init();

	Chip_Clock_EnablePeriphClock(SYSCON_CLOCK_OTP);
	otpApi->otpEnableBankReadLock(OTP_ROM_BANK, 0x03, 0, 0);
	ecrp_otp_val = Chip_OTP_ReadBankRegister(OTP_ROM_BANK, OTP_ECRP_REG);
	secure_otp_val = Chip_OTP_ReadBankRegister(OTP_ROM_BANK, OTP_SECURE_REG);

	DEBUGOUT("ECRP Data in OTP = 0x%08x\r\n", ecrp_otp_val);
	DEBUGOUT("Security Data in OTP (Current Value) = 0x%08x\r\n", secure_otp_val);

#ifdef I_KNOW_WHAT_IAM_DOING /* WARNING: DO NOT RUN THE BELOW CODE UNLESS YOU REALLY WANT TO WRITE TO OTP!!! */
#warning "OTP Programming of fuses enabled!! Make sure you know what you are doing!!!!!"
	otpApi->otpEnableBankWriteMask(1 << OTP_ROM_BANK);
	otpApi->otpProgramReg(OTP_ROM_BANK, OTP_SECURE_REG, 0xA5A5);
	secure_otp_val = Chip_OTP_ReadBankRegister(OTP_ROM_BANK, OTP_SECURE_REG);
	DEBUGOUT("Security Data in OTP (After Programming) = %x\r\n", secure_otp_val);
#else
	DEBUGSTR("OTP Fuse programming is not enabled!");
#endif


	while (loop) {
		__WFI();
	}
	return 0;
}
