/*
**************************************************************************************************************
*                                                 NXP USB Host Stack
*
*                                     (c) Copyright 2008, NXP SemiConductors
*                                     (c) Copyright 2008, OnChip  Technologies LLC
*                                                 All Rights Reserved
*
*                                                  www.nxp.com
*                                               www.onchiptech.com
*
* File           : usbhost_uart.c
* Programmer(s)  : Prasad.K.R.S.V
* Version        :
*
**************************************************************************************************************
*/

/*
**************************************************************************************************************
*                                           INCLUDE HEADER FILES
**************************************************************************************************************
*/

#include "usbhost_uart.h"

LPC_USART_T *pUART = DEBUG_UART;

/*
**************************************************************************************************************
*                                         INITIALIZE UART
*
* Description: This function initializes UART port, setup pin select, clock, parity, stopbits, FIFO etc
*
* Arguments  : baud_rate    UART baud rate (115200)
*
* Returns    : None
*
**************************************************************************************************************
*/

void  UART_Init(uint32_t baudrate)
{
	/* set "#define DEBUG_ENABLE" in board.h, then, for debugging print out of
	   the USB HostLIte example, the LPC5460x UART driver from chip and board library is used. */
	/* if DEBUG_ENABLE is not defined, invoke below API, baudrate will use DEBUGBAUDRATE. */
#ifndef DEBUG_ENABLE
// 	Board_UART_Init();
#endif
}

/*
**************************************************************************************************************
*                                         PRINT STRING
*
* Description: This function is used to print a string
*
* Arguments  : str    Pointer to the string
*
* Returns    : None
*
**************************************************************************************************************
*/

void  UART_PrintStr (uint8_t *str) {
	
  while (*str != 0) {
		if (*str == '\n') {
			if ( Chip_UART_GetFIFOStatus(pUART) & UART_FIFOSTAT_TXNOTFULL ) {
				Chip_UART_SendByte(pUART, *str++);
			}
			if ( Chip_UART_GetFIFOStatus(pUART) & UART_FIFOSTAT_TXNOTFULL ) {
				Chip_UART_SendByte(pUART, '\r');
      } 
		}
		else {
			if ( Chip_UART_GetFIFOStatus(pUART) & UART_FIFOSTAT_TXNOTFULL ) {
				Chip_UART_SendByte(pUART, *str++);
			}
    }    
	}
}

/*
**************************************************************************************************************
*                                        PRINT FORMATTED STRING
*
* Description: This function is used to print formatted string. This function takes variable length arguments
*
* Arguments  : variable length arguments
*
* Returns    : None
*
**************************************************************************************************************
*/

void  UART_Printf (const uint8_t *format, ...)
{
    static  uint8_t  buffer[40 + 1];
            va_list     vArgs;

    va_start(vArgs, format);
    vsprintf((char *)buffer, (char const *)format, vArgs);
    va_end(vArgs);
    UART_PrintStr((uint8_t *) buffer);
}
